import sys
from os import path


def infer_out_format(file_name):
    out_name, out_ext = path.splitext(file_name)
    return out_ext[1:].upper()


def get_samtools_opts(
    snakemake,
    parse_threads=True,
    parse_ref=True,
    parse_write_index=True,
    parse_output=True,
    parse_output_format=True,
):
    """Obtain samtools_opts from output, params, and handle resource definitions in resources."""
    samtools_opts = ""
    extra = snakemake.params.get("extra", "")
    idx = snakemake.output.get("idx", "")

    ###############
    ### Threads ###
    ###############
    if parse_threads:
        if "-@" in extra or "--threads" in extra:
            sys.exit(
                "You have specified number of threads (`-@/--threads`) in params.extra; please use only `threads`."
            )
        samtools_opts += (
            ""
            if snakemake.threads <= 1
            else " --threads {}".format(snakemake.threads - 1)
        )

    ######################
    ### Reference file ###
    ######################
    if parse_ref:
        if "--reference" in extra:
            sys.exit(
                "You have specified reference file (`--reference`) in `params.extra`; this is automatically infered from the `ref` input file."
            )

        if snakemake.input.get("ref"):
            samtools_opts += f" --reference {snakemake.input.ref}"

    ###################
    ### Write index ###
    ###################
    if parse_write_index:
        if "--write-index" in extra:
            sys.exit(
                "You have specified writing index (`--write-index`) in params.extra; this is automatically infered from `idx` output file."
            )
        if idx:
            samtools_opts += " --write-index"

    ###################
    ### Output file ###
    ###################
    if parse_output:
        if "-o" in extra:
            sys.exit(
                "You have specified output file (`-o`) in params.extra; this is automatically infered from the first output file."
            )
        samtools_opts += f" -o {snakemake.output[0]}"
        if idx:
            samtools_opts += f"##idx##{idx}"

    #####################
    ### Output format ###
    #####################
    if parse_output_format:
        if "-O" in extra or "--output-fmt" in extra:
            sys.exit(
                "You have specified output format (`-O/--output-fmt`) in params.extra; this is automatically infered from output file extension."
            )
        out_ext = infer_out_format(snakemake.output[0])
        samtools_opts += f" --output-fmt {out_ext}"

    return samtools_opts



import sys

def java_mem_xmx_error(unit, params_key):
    return f"You have specified resources.mem_{unit} and provided `-Xmx` in params.{params_key}. For Java memory specifications, please only use resources.mem_mb (for total memory reserved for the rule) and params.java_mem_overhead_mb (to specify any required non-heap overhead that needs to be set aside before determining the -Xmx value)."

def get_java_opts(snakemake, java_mem_overhead_factor=0.2):
    """Obtain java_opts from params, and handle resource definitions in resources."""

    java_opts = snakemake.params.get("java_opts", "")
    extra = snakemake.params.get("extra", "")
    assert 0.0 <= java_mem_overhead_factor <= 1.0

    # Getting memory in megabytes, if java opts is not filled with -Xmx parameter
    # By doing so, backward compatibility is preserved
    if "mem_mb" in snakemake.resources.keys():
        if "-Xmx" in java_opts:
            sys.exit(
                java_mem_xmx_error("mb", "java_opts")
            )
        if "-Xmx" in extra:
            sys.exit(
                java_mem_xmx_error("mb", "extra")
            )
        java_opts += " -Xmx{}M".format( round( snakemake.resources["mem_mb"] * (1.0 - java_mem_overhead_factor) ) )


    # Getting memory in gigabytes, for user convenience. Please prefer the use
    # of mem_mb over mem_gb as advised in documentation.
    elif "mem_gb" in snakemake.resources.keys():
        if "-Xmx" in java_opts:
            sys.exit(
                java_mem_xmx_error("gb", "java_opts")
            )
        if "-Xmx" in extra:
            sys.exit(
                java_mem_xmx_error("gb", "extra")
            )
        java_opts += " -Xmx{}G".format( round( snakemake.resources["mem_gb"] * (1.0 - java_mem_overhead_factor) ) )


    # Getting java temp directory from output files list, if -Djava.io.tmpdir
    # is not provided in java parameters. By doing so, backward compatibility is
    # not broken.
    if "java_temp" in snakemake.output.keys():
        if "-Djava.io.tmpdir" in java_opts:
            sys.exit(
                "You have specified output.java_temp and provided `-Djava.io.tmpdir` in params.java_opts. Please choose the one you intended and remove the other specification."
            )
        if "-Djava.io.tmpdir" in extra:
            sys.exit(
                "You have specified output.java_temp and provided `-Djava.io.tmpdir` in params.extra. Please choose the one you intended and remove the other specification."
            )
        java_opts += " -Djava.io.tmpdir={}".format(snakemake.output["java_temp"])

    return java_opts



def get_mem(snakemake, out_unit="MiB"):
    """
    Obtain requested memory (from resources) and return in given units.
    If no memory resources found, return 0.
    """

    # Store memory in MiB
    mem_mb = snakemake.resources.get("mem_gb", 0) * 1024
    if not mem_mb:
        mem_mb = snakemake.resources.get("mem_mb", 0)

    if out_unit == "KiB":
        return mem_mb * 1024
    elif out_unit == "MiB":
        return mem_mb
    elif out_unit == "GiB":
        return mem_mb / 1024
    else:
        raise valueError("invalid output unit. Only KiB, MiB and GiB supported.")



import sys


def infer_out_format(output, uncompressed_bcf=False):
    if output.endswith(".vcf"):
        return "v"
    elif output.endswith(".vcf.gz"):
        return "z"
    elif output.endswith(".bcf"):
        if uncompressed_bcf:
            return "u"
        else:
            return "b"
    else:
        raise ValueError("invalid output file format ('.vcf', '.vcf.gz', '.bcf').")


def get_bcftools_opts(
    snakemake,
    parse_threads=True,
    parse_ref=True,
    parse_regions=True,
    parse_samples=True,
    parse_targets=True,
    parse_output=True,
    parse_output_format=True,
    parse_memory=True,
):
    """Obtain bcftools_opts from output, params, and handle resource definitions."""
    bcftools_opts = ""
    extra = snakemake.params.get("extra", "")

    ###############
    ### Threads ###
    ###############
    if parse_threads:
        if "--threads" in extra:
            sys.exit(
                "You have specified number of threads (`--threads`) in `params.extra`; please use `threads`."
            )
        bcftools_opts += (
            ""
            if snakemake.threads <= 1
            else "--threads {}".format(snakemake.threads - 1)
        )

    ######################
    ### Reference file ###
    ######################
    if parse_ref:
        if "-f" in extra or "--fasta-ref" in extra:
            sys.exit(
                "You have specified reference file (`-f/--fasta-ref`) in `params.extra`; this is automatically infered from the `ref` input file."
            )

        if snakemake.input.get("ref"):
            bcftools_opts += f" --fasta-ref {snakemake.input.ref}"

    ####################
    ### Regions file ###
    ####################
    if parse_regions:
        if "--regions-file" in extra or "-R" in extra:
            sys.exit(
                "You have specified regions file (`-R/--regions-file`) in `params.extra`; this is automatically infered from the `regions` input file."
            )

        if snakemake.input.get("regions"):
            bcftools_opts += f" --regions-file {snakemake.input.regions}"

    ####################
    ### Samples file ###
    ####################
    if parse_samples:
        if "-S" in extra or "--samples-file" in extra:
            sys.exit(
                "You have specified samples file (`-S/--samples-file`) in `params.extra`; this is automatically infered from the `samples` input file."
            )

        if snakemake.input.get("samples"):
            bcftools_opts += f" --samples-file {snakemake.input.samples}"

    ####################
    ### Targets file ###
    ####################
    if parse_targets:
        if "-T" in extra or "--targets-file" in extra:
            sys.exit(
                "You have specified targets file (`-T/--targets-file`) in `params.extra`; this is automatically infered from the `targets` input file."
            )

        if snakemake.input.get("targets"):
            bcftools_opts += f" --targets-file {snakemake.input.targets}"

    ###################
    ### Output file ###
    ###################
    if parse_output:
        if "-o" in extra or "--output" in extra:
            sys.exit(
                "You have specified output file (`-o/--output`) in `params.extra`; this is automatically infered from the first output file."
            )
        bcftools_opts += f" --output {snakemake.output[0]}"

    #####################
    ### Output format ###
    #####################
    if parse_output_format:
        if "-O" in extra or "--output-type" in extra:
            sys.exit(
                "You have specified output format (`-O/--output-type`) in `params.extra`; this is automatically infered from output file extension."
            )

        out_format = infer_out_format(
            snakemake.output[0], snakemake.params.get("uncompressed_bcf", False)
        )
        bcftools_opts += f" --output-type {out_format}"

    ##############
    ### Memory ###
    ##############
    if parse_memory:
        if "-m" in extra or "--max-mem" in extra:
            sys.exit(
                "You have provided `-m/--max-mem` in `params.extra`; please use `resources.mem_mb`."
            )
        # Getting memory in megabytes, as advised in documentation.
        if "mem_mb" in snakemake.resources.keys():
            bcftools_opts += " --max-mem {}M".format(snakemake.resources["mem_mb"])
        # Getting memory in gigabytes, for user convenience. Please prefer the use
        # of mem_mb over mem_gb as advised in documentation.
        elif "mem_gb" in snakemake.resources.keys():
            bcftools_opts += " --max-mem {}G".format(snakemake.resources["mem_gb"])

    ################
    ### Temp dir ###
    ################
    if "-T" in extra or "--temp-dir" in extra or "--temp-prefix" in extra:
        sys.exit(
            "You have provided `-T/--temp-dir/--temp-prefix` in `params.extra`; please use the `tmpdir` resource."
        )

    return bcftools_opts
